package com.usky.iot.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.iot.domain.PmProject;
import com.usky.iot.mapper.PmProjectMapper;
import com.usky.iot.service.PmProjectService;
import com.usky.common.mybatis.core.AbstractCrudService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 项目表 服务实现类
 * </p>
 *
 * @author fu
 * @since 2024-05-20
 */
@Service
public class PmProjectServiceImpl extends AbstractCrudService<PmProjectMapper, PmProject> implements PmProjectService {

    @Autowired
    private PmProjectMapper pmProjectMapper;

    @Override
    public void addProject(PmProject project) {

    }

    @Override
    public void delProject(PmProject project) {

    }

    @Override
    public void updateProject(PmProject project) {

    }

    /**
     * 添加工作报告下拉框
     * 查询当前租户所有项目
     *
     * @return 只返回项目id和名称
     */
    @Override
    public List<PmProject> queryProject() {
        LambdaQueryWrapper<PmProject> wrapper = Wrappers.lambdaQuery();
        wrapper.select(PmProject::getId, PmProject::getProjectName)
                .eq(PmProject::getTenantId, SecurityUtils.getTenantId())
                .eq(PmProject::getDelFlag, 0)
                .and(qw -> qw
                        .eq(PmProject::getProjectHead, SecurityUtils.getUserId())
                        .or()
                        .apply("FIND_IN_SET('" + SecurityUtils.getUserId() + "', project_member) > 0")
                )
                .in(PmProject::getProjectStatus, 1, 2, 3);
        List<PmProject> list = this.list(wrapper);
        return list;
    }

    /**
     * 分页
     *
     * @param projectName       项目名
     * @param projectType       类型
     * @param projectStatus     状态
     * @param projectAscription 项目归属类型
     * @param pageNum           页码
     * @param pageSize          页大小
     * @return
     */
    @Override
    public CommonPage<PmProject> projectList(String projectName, Integer projectType, Integer projectStatus, Integer projectAscription, Integer pageNum, Integer pageSize) {
        IPage<PmProject> page = new Page<>(pageNum, pageSize);
        LambdaQueryWrapper<PmProject> lambdaQuery = Wrappers.lambdaQuery();
        lambdaQuery.select(PmProject::getId, PmProject::getProjectName, PmProject::getStartTime, PmProject::getEndTime, PmProject::getProjectStatus,
                PmProject::getProjectType, PmProject::getProjectHead, PmProject::getProjectMember)
                .eq(PmProject::getTenantId, SecurityUtils.getTenantId())
                .eq(PmProject::getDelFlag, 0)
                .orderByDesc(PmProject::getCreateTime);
        if (projectAscription == 1) {
            lambdaQuery.eq(PmProject::getProjectHead, SecurityUtils.getUserId());
        }
        if (projectAscription == 2) {
            lambdaQuery.apply("FIND_IN_SET('" + SecurityUtils.getUserId() + "', project_member) > 0");
        }
        if (!projectName.isEmpty()) {
            lambdaQuery.like(PmProject::getProjectName, projectName);
        }
        if (projectType != 0) {
            lambdaQuery.eq(PmProject::getProjectType, projectType);
        }
        if (projectStatus != 0) {
            lambdaQuery.eq(PmProject::getProjectStatus, projectStatus);
        }
        page = this.page(page, lambdaQuery);
        if (page.getRecords() == null || page.getRecords().isEmpty()) {
            return new CommonPage<>(Collections.emptyList(), 0, pageSize, pageNum);
        }
        return new CommonPage<>(page.getRecords(), page.getTotal(), pageSize, pageNum);
    }

    /**
     * 查询项目名
     *
     * @param projectIds
     * @return
     */
    @Override
    public List<PmProject> projectName(List<Integer> projectIds) {
        LambdaQueryWrapper<PmProject> wrapper = Wrappers.lambdaQuery();
        wrapper.select(PmProject::getId, PmProject::getProjectName)
                .in(PmProject::getId, projectIds);
        return pmProjectMapper.selectList(wrapper);
    }

    @Override
    public Map<String, Integer> sum() {
        LambdaQueryWrapper<PmProject> lambdaQuery = Wrappers.lambdaQuery();
        lambdaQuery.eq(PmProject::getTenantId, SecurityUtils.getTenantId())
                .eq(PmProject::getDelFlag, 0);
        Integer all = this.count(lambdaQuery);

        LambdaQueryWrapper<PmProject> lambdaQuery1 = Wrappers.lambdaQuery();
        lambdaQuery1.eq(PmProject::getTenantId, SecurityUtils.getTenantId())
                .eq(PmProject::getDelFlag, 0)
                .eq(PmProject::getProjectHead, SecurityUtils.getUserId());
        Integer head = this.count(lambdaQuery1);

        LambdaQueryWrapper<PmProject> lambdaQuery2 = Wrappers.lambdaQuery();
        lambdaQuery2.eq(PmProject::getTenantId, SecurityUtils.getTenantId())
                .eq(PmProject::getDelFlag, 0)
                .apply("FIND_IN_SET('" + SecurityUtils.getUserId() + "', project_member) > 0");
        Integer join = this.count(lambdaQuery2);

        Map<String, Integer> sumMap = new HashMap<>();
        sumMap.put("allProject",all);
        sumMap.put("headProject", head);
        sumMap.put("joinProject", join);
        return sumMap;
    }

}
